require 'slop'

gem_name = File.basename __FILE__, '.rb'

def assuming_at_least(nr, args, opts)
  valid = args.size >= nr
  yield if valid && block_given?
  puts opts unless valid
end

opts = Slop.new help: true do
  banner "Usage: #{gem_name} [command]"

  command 'dummy-scrape' do
    run do |opts, args|
      require 'tamiyo/dummy_scrape_chain'
      Tamiyo::DummyScrapeChain.create.pump
    end
  end

  command 'gatherer-scrape' do
    run do |opts, args|
      set = args.first || "Innistrad"
      require 'tamiyo/gatherer_scrape_chain'
      Tamiyo::GathererScrapeChain.create(set).pump
    end
  end

  command 'export-all' do
    run do |opts, args|
      require 'tamiyo/export_all_chain'
      Tamiyo::ExportAllChain.create.pump
    end
  end

  command 'add-set' do
    banner "Usage: #{gem_name} add-set [abbr] [set name]"
    run do |opts, args|
      assuming_at_least(2, args, opts) do
        abbr = args.shift.upcase
        name = args.join(' ')
        puts "Adding set #{abbr} => #{name}"
        require 'tamiyo/sets'
        Tamiyo::Sets.add_custom_set abbr, name
      end
    end
  end

  command 'remove-set' do
    banner "Usage: #{gem_name} remove-set [ [abbr] | --name [set name] ]"
    on :n, :name, 'Remove set by full name instead of abbrevation'

    run do |opts, args|
      assuming_at_least(1, args, opts) do
        name = opts.name? ? args.join(' ') : args.shift.upcase
        puts "Removing set #{name}"
        require 'tamiyo/sets'
        Tamiyo::Sets.remove_custom_set name
      end
    end
  end
end

opts.parse
puts opts if ARGV.empty?
