require 'nokogiri'
require 'tamiyo/process'
require 'tamiyo/data_store'
require 'tamiyo/default_data'

module Tamiyo
  class CockatriceDbSink
    include Process::Node

    def pump
      content = Nokogiri::XML::Builder.new(encoding: 'UTF-8') do |xml|
        xml.cockatrice_carddatabase(version: '2') {
          add_sets_to xml
          add_cards_to xml
        }
      end
      store_xml content
    end

    def add_sets_to(xml)
      xml.sets {
        DefaultData.sets['sets'].each do |abbr, name|
          xml.set {
            xml.name abbr
            xml.longname name
          }
        end
      }
    end

    def add_cards_to(xml)
      xml.cards {
        @chain.pull.each do |card|
          xml.card {
            xml.name card.name
            # sets
            card.colors.each { |color| xml.color color }
            xml.manacost first_cost_of card
            xml.type_ card.type
            xml.pt card.pt if card.pt
            xml.loyalty card.loyalty if card.loyalty
            xml.tablerow row_of card
            xml.text_ adjusted_text_of card
            xml.cipt '1' if card.played_tapped
          }
        end
      }
    end

    def store_xml(content)
      data_store = DataStore.new
      path = data_store.path_for 'cards.xml'
      File.open(path, 'w') do |file|
        file.write content.to_xml
      end
    end

    def first_cost_of(card)
      [*card.cost].first
    end

    def row_of(card)
      case card.table_row
      when :first; '0'
      when :second; '1'
      when :third; '2'
      when :stack; '3'
      end
    end

    def adjusted_text_of(card)
      text = card.text
      if card.cost.is_a? Array
        first, second = card.cost
        text = "Mana cost: #{first}\n#{text}"
        text.sub!(/---\n/) { |m| m << "Mana cost: #{second}\n" }
      end
      text
    end
  end
end