require 'fileutils'
require 'tamiyo/yaml/yaml_helper'

module Tamiyo
  class DataStore
    include YamlHelper

    def initialize
      @data_dir = ensure_tamiyo_data_dir
      @custom_sets_path = path_for 'custom_sets.yaml'
    end

    def ensure_tamiyo_data_dir
      data_dir = File.join Dir.home, '.tamiyo'
      FileUtils.mkpath data_dir
      data_dir
    end

    def path_for(file_name)
      File.join @data_dir, file_name
    end

    def load_custom_sets
      File.exists?(@custom_sets_path) ? begin
        File.open(@custom_sets_path, 'r') do |f|
          yaml_from f.read
        end
      end : {}
    end

    def persist_custom_sets(data)
      File.open(@custom_sets_path, 'w') do |f|
        f.write yaml_of data
      end
    end
  end
end