require 'tamiyo/process'
require 'tamiyo/data_store'
require 'tamiyo/card'
require_relative 'yaml_helper'

module Tamiyo
  module Yaml
    class YamlSource
      include Process::Node
      include YamlHelper

      def pull
        using_object [] do |cards|
          with_the_cards_file do |events|
            events.each_within_sequence do
              proto = using_object(Prototype.new) do |proto|
                events.each_within_mapping do
                  parse_card_data_from(events, proto)
                end
              end
              cards << proto.build_card
            end
          end
        end
      end

      def parse_card_data_from(events, proto)
        events.on_keys(
          'name' => ->{ proto.name = events.assume_scalar },
          'identity' => ->{
            events.each_within_sequence do
              proto.colors << events.assume_scalar
            end },
          'mana cost' => ->{
            events.on_types(
              scalar: ->{ proto.cost = events.value },
              seq: ->{
                events.each_within_sequence do
                  proto.cost ||= []
                  proto.cost << events.assume_scalar
                end }) },
          'type' => ->{ proto.type = events.assume_scalar },
          'text' => ->{ proto.text = events.assume_scalar },
          'pt' => ->{ proto.pt = events.assume_scalar },
          'loyalty' => ->{ proto.loyalty = events.assume_scalar },
          'row' => ->{ proto.table_row = events.assume_scalar.to_sym },
          'played tapped' => ->{
            events.assume_scalar
            proto.played_tapped = true },
          'picture url' => ->{ proto.picture_url = events.assume_scalar })
      end

      def with_the_cards_file
        data_store = DataStore.new
        path = data_store.path_for 'cards.yaml'
        if block_given?
          File.open(path, 'r') do |file|
            yield yaml_event_stream_for file
          end
        end
      end

      def using_object(obj)
        yield obj
        obj
      end

      Prototype = Struct.new(:name, :cost, :colors, :type, :text, :pt, :loyalty,
                             :table_row, :played_tapped, :picture_url) do
        def initialize
          self.colors = []
          self.played_tapped = false
          self.picture_url = ''
        end

        def build_card
          Card.new name, cost, colors, type, text, pt, loyalty,
                   table_row, played_tapped, picture_url
        end
      end
    end
  end
end